# coding=utf-8
import requests
from bs4 import BeautifulSoup
import re
import dimjson

URL = 'https://filmux.net'
SEARCH_URL = "/index.php?do=search"


def process_page_paging(soup):
    next_data = {}
    nextel = soup.find(class_=['page-next'])
    if not nextel:
        return None
    a = nextel.find('a')
    if not a:
        return None
    next_data['next_url'] = a.get('href')
    nextel = soup.find(class_=['pages'])
    if not nextel:
        return None
    try:
        for span in nextel.find_all('span'):
            value = __t(span)
            if value.isdigit():
                next_data['current'] = int(value)
        next_data['total'] = int(__t(nextel.find_all('a')[-1], '0'))
        return next_data
    except ValueError:
        return None


def __make_soup(url, params=None):
    if url.startswith('/'):
        url = URL + url
    req = requests.get(url, params=params)
    req.raise_for_status()
    soup = BeautifulSoup(req.text, "html.parser")
    return soup


def __get_img_url(img):
    if not img:
        return None
    img_src = img.get('data-src', img.get('src'))
    if img_src:
        if img_src.startswith('/'):
            img_src = URL + img_src
        return img_src
    return None


def __t(element, default=''):
    if element is None:
        return default
    text = element.string
    if not text:
        return default
    return text.strip()


def __proc_js_obj(value):
    if not value:
        return
    result = []
    f = value.get('file')
    if not f:
        return
    img = value.get('poster')
    name = value.get('comment')
    x = re.findall(r'((http:|https:|)//([\w_-]+(?:(?:\.[\w_-]+)+))([\w.,@?^=%&:/~+#-]*[\w@?^=%&/~+#-])?)', f)
    urls = set([url[0] for url in x])
    for url in urls:
        if url.startswith('//'):
            # Lets just assume https
            url = 'https:' + url
        if not url.startswith('http://') and not url.startswith('https://'):
            continue
        result.append({
            'url': url,
            'img': img,
            'name': name,
        })
    return result


def process_video_source(url):
    if not url:
        return
    soup = __make_soup(url)
    scripts = soup.find_all('script')
    for script in scripts:
        text = __t(script)
        index = -1
        if 'new Playerjs(' in text:
            index = text.index('new Playerjs(') + len('new Playerjs(')
        if 'new Uppod(' in text:
            index = text.index('new Uppod(') + len('new Uppod(')
        if index < 0:
            continue
        js_argument = text[index:text.index('});', index)]
        js_obj = dimjson.decode(js_argument + '}')
        # remove [360p] from start of URL
        result = []
        if isinstance(js_obj.get('file'), list):
            for value in js_obj.get('file'):
                sources = __proc_js_obj(value)
                if sources:
                    result += sources
        else:
            sources = __proc_js_obj(js_obj)
            if sources:
                result += sources
        return result


def process_video(url):
    soup = __make_soup(url)
    content = soup.find(id='dle-content') or soup
    film = {}
    ftitle = content.find(class_=['f-title'])
    if ftitle:
        film['title'] = __t(ftitle.find('span'), None)
    film['description'] = __t(content.find(class_=['f-text']), None)
    film['img'] = __get_img_url(content.find('img'))
    film['sources'] = []
    for iframe in content.find_all('iframe'):
        src = iframe.get('src')
        if 'youtube' in src:
            film['trailer'] = src
        else:
            sources = process_video_source(src)
            if sources:
                film['sources'] += sources
    film['similar'] = __process_page(soup)
    return film


def process_page(url):
    soup = __make_soup(url)
    return __process_page(soup)


def __process_page(soup):
    result = []
    content = soup.find(id='dle-content') or soup
    for a in content.find_all('a'):
        film = {'url': a.get('href')}
        strong = a.find('strong')
        film['title'] =  a.get('title')if strong is None else strong.text
        if not film['url'] or not film['title'] or \
                not re.match(r'^https:\/\/filmux\.net\/[a-zA-Z0-9\-_]+\.html$', film['url']):
            continue
        film['img'] = __get_img_url(a.parent.find('img'))
        film['description'] = __t(a.parent.parent.find(class_=['short-text']), None)
        fields = a.parent.parent.find(class_=['short-fields'])
        if fields:
            for p in fields.find_all('p'):
                em = __t(p.find('em'))
                span = __t(p.find('span'))
                if em == u'Pridėta:':
                    film['added'] = span
                elif em == u'Žanras:':
                    film['genre'] = span
                elif em == u'Metai:':
                    film['year'] = span
                elif em == u'Kalba:':
                    film['language'] = span
                elif em == u'Aktoriai:':
                    film['actors'] = span
        result.append(film)
    return result, process_page_paging(soup)


def search(name, page=0):
    soup = __make_soup(SEARCH_URL, {
        "do": "search",
        "subaction": "search",
        "search_start": page,
        "full_search": "0",
        # "result_from": "1",
        "story": name
    })
    return __process_page(soup)


if __name__ == '__main__':
    # data = process_page(u'https://filmux.net/10826-ispudingas-viesbutis-2-sezonas.html')
    data = process_video(u'https://filmux.net/11093-sobiboras.html')
    # data = process_video('https://filmux.net/464-hercogiene.html')
    # data = search('piratai')
    # data2 = search('filmas', 2)
    pass
